@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -117,8 +117,7 @@ ip2cidr(const unsigned long ip, const int masklen)
 
     network = (u_char *)safe_malloc(20);
 
-    strlcpy((char *)network, (char *)get_addr2name4(ip, RESOLVE),
-            sizeof(network));
+    strlcpy((char *)network, (char *)get_addr2name4(ip, RESOLVE), 20);
 
     strcat((char *)network, "/");
     if (masklen < 10) {
@@ -342,6 +341,7 @@ parse_endpoints(tcpr_cidrmap_t ** cidrmap1, tcpr_cidrmap_t ** cidrmap2, const ch
     char *token = NULL;
     char *string;
     char *p;
+    int res = 0;
 
     string = safe_strdup(optarg);
 
@@ -350,7 +350,7 @@ parse_endpoints(tcpr_cidrmap_t ** cidrmap1, tcpr_cidrmap_t ** cidrmap2, const ch
         memset(newmap, '\0', NEWMAP_LEN);
         p = strstr(string, "]:[");
         if (!p)
-            return 0;
+            goto done;
             
         *p = 0;
         strlcpy(newmap, "[::/0]:", NEWMAP_LEN);
@@ -358,7 +358,7 @@ parse_endpoints(tcpr_cidrmap_t ** cidrmap1, tcpr_cidrmap_t ** cidrmap2, const ch
         strlcat(newmap, "]", NEWMAP_LEN);
         
         if (! parse_cidr_map(cidrmap1, newmap))
-            return 0;
+            goto done;
 
         /* do again with the second IP */
         memset(newmap, '\0', NEWMAP_LEN);
@@ -366,7 +366,7 @@ parse_endpoints(tcpr_cidrmap_t ** cidrmap1, tcpr_cidrmap_t ** cidrmap2, const ch
         strlcat(newmap, p + 2, NEWMAP_LEN);
 
         if (! parse_cidr_map(cidrmap2, newmap))
-            return 0;
+            goto done;
 
     } else {
         /* ipv4 mode */
@@ -376,7 +376,7 @@ parse_endpoints(tcpr_cidrmap_t ** cidrmap1, tcpr_cidrmap_t ** cidrmap2, const ch
         strlcpy(newmap, "0.0.0.0/0:", NEWMAP_LEN);
         strlcat(newmap, map, NEWMAP_LEN);
         if (! parse_cidr_map(cidrmap1, newmap))
-            return 0;
+            goto done;
     
         /* do again with the second IP */
         memset(newmap, '\0', NEWMAP_LEN);
@@ -385,11 +385,15 @@ parse_endpoints(tcpr_cidrmap_t ** cidrmap1, tcpr_cidrmap_t ** cidrmap2, const ch
         strlcpy(newmap, "0.0.0.0/0:", NEWMAP_LEN);
         strlcat(newmap, map, NEWMAP_LEN);
         if (! parse_cidr_map(cidrmap2, newmap))
-            return 0;
+            goto done;
     }
     
+    /* success */
+    res = 1;
+
+done:
     safe_free(string);
-    return 1; /* success */
+    return res;
 }
 
 
@@ -407,17 +411,18 @@ parse_cidr_map(tcpr_cidrmap_t **cidrmap, const char *optarg)
     char *map = NULL;
     char *token = NULL, *string = NULL;
     tcpr_cidrmap_t *ptr;
+    int res = 0;
     
     string = safe_strdup(optarg);
 
     /* first iteration */
     map = strtok_r(string, ",", &token);
     if (! parse_cidr(&cidr, map, ":"))
-        return 0;
+        goto done;
 
     /* must return a linked list of two */
     if (cidr->next == NULL)
-        return 0;
+        goto done;
 
     /* copy over */
     *cidrmap = new_cidr_map();
@@ -434,11 +439,11 @@ parse_cidr_map(tcpr_cidrmap_t **cidrmap, const char *optarg)
             break;
 
         if (! parse_cidr(&cidr, map, ":"))
-            return 0;
+            goto done;
 
         /* must return a linked list of two */
         if (cidr->next == NULL)
-            return 0;
+            goto done;
 
         /* copy over */
         ptr->next = new_cidr_map();
@@ -447,9 +452,13 @@ parse_cidr_map(tcpr_cidrmap_t **cidrmap, const char *optarg)
         ptr->to = cidr->next;
         ptr->from->next = NULL;
     }
-    
+
+    /* success */
+    res = 1;
+
+done:
     safe_free(string);
-    return 1; /* success */
+    return res;
 }
 
 /**
