@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -597,7 +597,7 @@ sendpacket_getstat(sendpacket_t *sp, char *buf, size_t buf_size)
 /**
  * close the given sendpacket
  */
-int
+void
 sendpacket_close(sendpacket_t *sp)
 {
     assert(sp);
@@ -657,7 +657,6 @@ sendpacket_close(sendpacket_t *sp)
             break;
     }
     safe_free(sp);
-    return 0;
 }
 
 /**
@@ -840,10 +839,11 @@ sendpacket_open_tuntap(const char *device, char *errbuf)
     }
     memset(&ifr, 0, sizeof(ifr));
     ifr.ifr_flags = (IFF_TAP | IFF_NO_PI);
-    strncpy(ifr.ifr_name, device, strlen(device));
+    strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name)-1);
 
     if (ioctl(tapfd, TUNSETIFF, (void *) &ifr) < 0) {
         snprintf(errbuf, SENDPACKET_ERRBUF_SIZE, "Unable to create tuntap interface: %s", device);
+        close(tapfd);
         return NULL;
     }
 #elif defined(HAVE_FREEBSD)
@@ -903,6 +903,8 @@ sendpacket_open_pf(const char *device, char *errbuf)
     dbg(1, "sendpacket: using PF_PACKET");
 #endif
 
+    memset(&sa, 0, sizeof(sa));
+
     /* open our socket */
     if ((mysocket = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) {
         snprintf(errbuf, SENDPACKET_ERRBUF_SIZE, "socket: %s", strerror(errno));
