@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -127,6 +127,7 @@ struct sendpacket_s {
     struct tcpr_ether_addr ether;
 #if defined HAVE_QUICK_TX || defined HAVE_NETMAP
     int first_packet;
+    int netmap_delay;
 #endif
 
 #ifdef HAVE_QUICK_TX
@@ -164,7 +165,7 @@ struct sendpacket_s {
 typedef struct sendpacket_s sendpacket_t;
 
 int sendpacket(sendpacket_t *, const u_char *, size_t, struct pcap_pkthdr *);
-int sendpacket_close(sendpacket_t *);
+void sendpacket_close(sendpacket_t *);
 char *sendpacket_geterr(sendpacket_t *);
 size_t sendpacket_getstat(sendpacket_t *, char *, size_t);
 sendpacket_t *sendpacket_open(const char *, char *, tcpr_dir_t, sendpacket_type_t, void *arg);
