@@ -3,7 +3,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -69,7 +69,7 @@ extern tcpedit_t *tcpedit;
 extern int debug;
 #endif
 
-static void calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
+static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
         struct timeval *last, COUNTER len,
         sendpacket_t *sp, COUNTER counter, timestamp_t *sent_timestamp,
         COUNTER *start_us, COUNTER *skip_length);
@@ -102,7 +102,6 @@ fast_edit_packet_dl(struct pcap_pkthdr *pkthdr, u_char **pktdata,
     hdlc_hdr_t *hdlc_hdr;
     sll_hdr_t *sll_hdr;
     struct tcpr_pppserial_hdr *ppp;
-    uint32_t *src_ptr = NULL, *dst_ptr = NULL;
     uint32_t src_ip, dst_ip;
     uint32_t src_ip_orig, dst_ip_orig;
     uint16_t ether_type = 0;
@@ -230,9 +229,6 @@ fast_edit_packet_dl(struct pcap_pkthdr *pkthdr, u_char **pktdata,
     }
 
     dbgx(1, "(%u): final src_ip=0x%08x dst_ip=0x%08x", iteration, src_ip, dst_ip);
-
-    *src_ptr = htonl(src_ip);
-    *dst_ptr = htonl(dst_ip);
 }
 
 #if defined HAVE_QUICK_TX || defined HAVE_NETMAP
@@ -463,6 +459,19 @@ preload_pcap_file(tcpreplay_t *ctx, int idx)
     pcap_close(pcap);
 }
 
+static void increment_iteration(tcpreplay_t *ctx)
+{
+    tcpreplay_opt_t *options = ctx->options;
+
+    ctx->last_unique_iteration = ctx->unique_iteration;
+    ++ctx->iteration;
+    if (options->unique_ip) {
+        assert(options->unique_loops > 0.0);
+        ctx->unique_iteration =
+                (COUNTER)((float)ctx->iteration / options->unique_loops) + 1;
+    }
+}
+
 /**
  * the main loop function for tcpreplay.  This is where we figure out
  * what to do with each packet
@@ -470,7 +479,8 @@ preload_pcap_file(tcpreplay_t *ctx, int idx)
 void
 send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
 {
-    struct timeval print_delta, now;
+
+    struct timeval print_delta, now, first_pkt_ts, pkt_ts_delta;
     tcpreplay_opt_t *options = ctx->options;
     COUNTER packetnum = 0;
     COUNTER limit_send = options->limit_send;
@@ -487,14 +497,14 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
     COUNTER skip_length = 0;
     COUNTER start_us;
     COUNTER end_us;
-    COUNTER iteration = ctx->iteration;
-    bool unique_ip = options->unique_ip;
     bool preload = options->file_cache[idx].cached;
-    bool top_speed = (options->speed.mode == speed_topspeed);
-    bool now_is_now = false;
+    bool top_speed = (options->speed.mode == speed_topspeed ||
+            (options->speed.mode == speed_mbpsrate && options->speed.speed == 0));
+    bool now_is_now;
 
     ctx->skip_packets = 0;
     start_us = TIMEVAL_TO_MICROSEC(&ctx->stats.start_time);
+    timerclear(&first_pkt_ts);
 
     if (options->limit_time > 0)
         end_us = start_us + SEC_TO_MICROSEC(options->limit_time);
@@ -507,6 +517,13 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
         prev_packet = NULL;
     }
 
+    if (!top_speed) {
+        gettimeofday(&now, NULL);
+        now_is_now = true;
+    } else {
+        now_is_now = false;
+    }
+
     /* MAIN LOOP 
      * Keep sending while we have packets or until
      * we've sent enough packets
@@ -514,6 +531,7 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
     while (!ctx->abort &&
             (pktdata = get_next_packet(ctx, pcap, &pkthdr, idx, prev_packet)) != NULL) {
 
+        now_is_now = false;
         packetnum++;
 #if defined TCPREPLAY || defined TCPREPLAY_EDIT
         /* do we use the snaplen (caplen) or the "actual" packet len? */
@@ -550,16 +568,24 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
             tcpdump_print(options->tcpdump, &pkthdr, pktdata);
 #endif
 
-        if (unique_ip && iteration)
+        if (ctx->options->unique_ip && ctx->unique_iteration &&
+                ctx->unique_iteration > ctx->last_unique_iteration) {
             /* edit packet to ensure every pass has unique IP addresses */
-            fast_edit_packet(&pkthdr, &pktdata, iteration,
+            fast_edit_packet(&pkthdr, &pktdata, ctx->unique_iteration - 1,
                     preload, datalink);
+        }
 
         /* update flow stats */
         if (options->flow_stats && !preload)
             update_flow_stats(ctx,
                     options->cache_packets ? sp : NULL, &pkthdr, pktdata, datalink);
 
+        if (ctx->first_time) {
+            /* get time and timestamp of the first packet */
+            gettimeofday(&now, NULL);
+            memcpy(&first_pkt_ts, &pkthdr.ts, sizeof(struct timeval));
+        }
+
         /*
          * this accelerator improves performance by avoiding expensive
          * time stamps during periods where we have fallen behind in our
@@ -567,28 +593,40 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
          */
         if (skip_length && pktlen < skip_length) {
             skip_length -= pktlen;
-            now_is_now = false;
         } else if (ctx->skip_packets) {
             --ctx->skip_packets;
-            now_is_now = false;
         } else {
             /*
              * time stamping is expensive, but now is the
              * time to do it.
              */
+            dbgx(4, "This packet time: " TIMEVAL_FORMAT, pkthdr.ts.tv_sec, pkthdr.ts.tv_usec);
             skip_length = 0;
             ctx->skip_packets = 0;
-            now_is_now = true;
-            gettimeofday(&now, NULL);
 
             /*
              * Only sleep if we're not in top speed mode (-t)
+             * or if the current packet is not late.
              *
              * This also sets skip_length which will avoid timestamping for
              * a given number of packets.
              */
-            calc_sleep_time(ctx, &pkthdr.ts, &ctx->stats.last_time, pktlen, sp, packetnum,
-                    &ctx->stats.end_time, &start_us, &skip_length);
+            timersub(&pkthdr.ts, &first_pkt_ts, &pkt_ts_delta);
+            if (calc_sleep_time(ctx, &pkt_ts_delta, &ctx->stats.last_time, pktlen, sp, packetnum,
+                    &ctx->stats.end_time, &start_us, &skip_length)) {
+                now_is_now = true;
+                gettimeofday(&now, NULL);
+            }
+
+            /*
+             * track the time of the "last packet sent".  Again, because of OpenBSD
+             * we have to do a memcpy rather then assignment.
+             *
+             * A number of 3rd party tools generate bad timestamps which go backwards
+             * in time.  Hence, don't update the "last" unless pkthdr.ts > last
+             */
+            if (!top_speed && timercmp(&ctx->stats.last_time, &pkt_ts_delta, <)) 
+                memcpy(&ctx->stats.last_time, &pkt_ts_delta, sizeof(struct timeval));
 
             /*
              * we know how long to sleep between sends, now do it.
@@ -614,15 +652,6 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
 #ifdef TIMESTAMP_TRACE
         add_timestamp_trace_entry(pktlen, &ctx->stats.end_time, skip_length);
 #endif
-        /*
-         * track the time of the "last packet sent".  Again, because of OpenBSD
-         * we have to do a memcpy rather then assignment.
-         *
-         * A number of 3rd party tools generate bad timestamps which go backwards
-         * in time.  Hence, don't update the "last" unless pkthdr.ts > last
-         */
-        if (!top_speed && timercmp(&ctx->stats.last_time, &pkthdr.ts, <)) 
-            memcpy(&ctx->stats.last_time, &pkthdr.ts, sizeof(struct timeval));
 
         ctx->stats.pkts_sent++;
         ctx->stats.bytes_sent += pktlen;
@@ -676,7 +705,7 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
 
     memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
 
-    ++ctx->iteration;
+    increment_iteration(ctx);
 }
 
 /**
@@ -686,7 +715,7 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
 void
 send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *pcap2, int cache_file_idx2)
 {
-    struct timeval print_delta, now;
+    struct timeval print_delta, now, first_pkt_ts, pkt_ts_delta;
     tcpreplay_opt_t *options = ctx->options;
     COUNTER packetnum = 0;
     COUNTER limit_send = options->limit_send;
@@ -695,8 +724,6 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
     u_char *pktdata1 = NULL, *pktdata2 = NULL, *pktdata = NULL;
     sendpacket_t *sp = ctx->intf1;
     COUNTER pktlen;
-    COUNTER iteration = ctx->iteration;
-    bool unique_ip = options->unique_ip;
     packet_cache_t *cached_packet1 = NULL, *cached_packet2 = NULL;
     packet_cache_t **prev_packet1 = NULL, **prev_packet2 = NULL;
     struct pcap_pkthdr *pkthdr_ptr;
@@ -704,11 +731,13 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
     COUNTER start_us;
     COUNTER end_us;
     COUNTER skip_length = 0;
-    bool top_speed = (options->speed.mode == speed_topspeed);
-    bool now_is_now = false;
+    bool top_speed = (options->speed.mode == speed_topspeed ||
+            (options->speed.mode == speed_mbpsrate && options->speed.speed == 0));
+    bool now_is_now;
 
     ctx->skip_packets = 0;
     start_us = TIMEVAL_TO_MICROSEC(&ctx->stats.start_time);
+    timerclear(&first_pkt_ts);
 
     if (options->limit_time > 0)
         end_us = start_us + SEC_TO_MICROSEC(options->limit_time);
@@ -727,6 +756,13 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
     pktdata1 = get_next_packet(ctx, pcap1, &pkthdr1, cache_file_idx1, prev_packet1);
     pktdata2 = get_next_packet(ctx, pcap2, &pkthdr2, cache_file_idx2, prev_packet2);
 
+    if (!top_speed) {
+        gettimeofday(&now, NULL);
+        now_is_now = true;
+    } else {
+        now_is_now = false;
+    }
+
     /* MAIN LOOP 
      * Keep sending while we have packets or until
      * we've sent enough packets
@@ -734,6 +770,7 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
     while (!ctx->abort &&
             !(pktdata1 == NULL && pktdata2 == NULL)) {
 
+        now_is_now = false;
         packetnum++;
 
         /* figure out which pcap file we need to process next 
@@ -797,15 +834,23 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
             tcpdump_print(options->tcpdump, pkthdr_ptr, pktdata);
 #endif
 
-        if (unique_ip && iteration)
+        if (ctx->options->unique_ip && ctx->unique_iteration &&
+                ctx->unique_iteration > ctx->last_unique_iteration) {
             /* edit packet to ensure every pass is unique */
-            fast_edit_packet(pkthdr_ptr, &pktdata, ctx->iteration,
+            fast_edit_packet(pkthdr_ptr, &pktdata, ctx->unique_iteration - 1,
                     options->file_cache[cache_file_idx].cached, datalink);
+        }
 
         /* update flow stats */
         if (options->flow_stats && !options->file_cache[cache_file_idx].cached)
             update_flow_stats(ctx, sp, pkthdr_ptr, pktdata, datalink);
 
+        if (ctx->first_time) {
+            /* get time and timestamp of the first packet */
+            gettimeofday(&now, NULL);
+            memcpy(&first_pkt_ts, &pkthdr_ptr->ts, sizeof(struct timeval));
+        }
+
         /*
          * this accelerator improves performance by avoiding expensive
          * time stamps during periods where we have fallen behind in our
@@ -813,28 +858,40 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
          */
         if (skip_length && pktlen < skip_length) {
             skip_length -= pktlen;
-            now_is_now = false;
         } else if (ctx->skip_packets) {
             --ctx->skip_packets;
-            now_is_now = false;
         } else {
             /*
              * time stamping is expensive, but now is the
              * time to do it.
              */
+            dbgx(4, "This packet time: " TIMEVAL_FORMAT, pkthdr_ptr->ts.tv_sec, pkthdr_ptr->ts.tv_usec);
             skip_length = 0;
             ctx->skip_packets = 0;
-            now_is_now = true;
-            gettimeofday(&now, NULL);
 
             /*
              * Only sleep if we're not in top speed mode (-t)
+             * or if the current packet is not late.
              *
              * This also sets skip_length which will avoid timestamping for
              * a given number of packets.
              */
-            calc_sleep_time(ctx, &pkthdr_ptr->ts, &ctx->stats.last_time, pktlen, sp, packetnum,
-                    &ctx->stats.end_time, &start_us, &skip_length);
+            timersub(&pkthdr_ptr->ts, &first_pkt_ts, &pkt_ts_delta);
+            if (calc_sleep_time(ctx, &pkt_ts_delta, &ctx->stats.last_time, pktlen, sp, packetnum,
+                    &ctx->stats.end_time, &start_us, &skip_length)) {
+                now_is_now = true;
+                gettimeofday(&now, NULL);
+            }
+
+            /*
+             * track the time of the "last packet sent".  Again, because of OpenBSD
+             * we have to do a memcpy rather then assignment.
+             *
+             * A number of 3rd party tools generate bad timestamps which go backwards
+             * in time.  Hence, don't update the "last" unless pkthdr.ts > last
+             */
+            if (!top_speed && timercmp(&ctx->stats.last_time, &pkt_ts_delta, <))
+                memcpy(&ctx->stats.last_time, &pkt_ts_delta, sizeof(struct timeval));
 
             /*
              * we know how long to sleep between sends, now do it.
@@ -857,16 +914,6 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
          */
         memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
 
-        /*
-         * track the time of the "last packet sent".  Again, because of OpenBSD
-         * we have to do a memcpy rather then assignment.
-         *
-         * A number of 3rd party tools generate bad timestamps which go backwards
-         * in time.  Hence, don't update the "last" unless pkthdr.ts > last
-         */
-        if (!top_speed && timercmp(&ctx->stats.last_time, &pkthdr_ptr->ts, <))
-            memcpy(&ctx->stats.last_time, &pkthdr_ptr->ts, sizeof(struct timeval));
-
         ctx->stats.pkts_sent++;
         ctx->stats.bytes_sent += pktlen;
 
@@ -926,7 +973,7 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
 
     memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
 
-    ++ctx->iteration;
+    increment_iteration(ctx);
 }
 
 
@@ -1061,14 +1108,15 @@ cache_mode(tcpreplay_t *ctx, char *cachedata, COUNTER packet_num)
  * calculate the appropriate amount of time to sleep. Sleep time
  * will be in ctx->nap.
  */
-static void calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
-        struct timeval *last, COUNTER len,
+static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time_delta,
+        struct timeval *last_delta, COUNTER len,
         sendpacket_t *sp, COUNTER counter, timestamp_t *sent_timestamp,
         COUNTER *start_us, COUNTER *skip_length)
 {
     tcpreplay_opt_t *options = ctx->options;
     struct timeval nap_for;
     COUNTER now_us;
+    bool update_time = true;
 
     timesclear(&ctx->nap);
 
@@ -1078,34 +1126,41 @@ static void calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
      */
     if (options->speed.mode == speed_packetrate && options->speed.pps_multi) {
         ctx->skip_packets = options->speed.pps_multi - 1;
-        if (ctx->first_time) {
-            ctx->first_time = false;
-            return;
-        }
     }
 
-    dbgx(4, "This packet time: " TIMEVAL_FORMAT, pkt_time->tv_sec, pkt_time->tv_usec);
-    dbgx(4, "Last packet time: " TIMEVAL_FORMAT, last->tv_sec, last->tv_usec);
+    /* no last packet sent, just leave */
+    if (ctx->first_time) {
+        ctx->first_time = false;
+        return false;
+    }
 
     switch(options->speed.mode) {
     case speed_multiplier:
         /*
          * Replay packets a factor of the time they were originally sent.
          */
-        if (timerisset(last)) {
-            if (timercmp(pkt_time, last, >=)) {
-                /* pkt_time has increased or is the same, so handle normally */
-                timersub(pkt_time, last, &nap_for);
+        if (timerisset(last_delta)) {
+            /* make sure the packet is not late */
+            COUNTER delta_us, delta_pkt_time;
+            now_us = TIMSTAMP_TO_MICROSEC(sent_timestamp);
+            delta_pkt_time = TIMEVAL_TO_MICROSEC(pkt_time_delta);
+            delta_us = now_us - *start_us;
+            if (timercmp(pkt_time_delta, last_delta, >) && (delta_pkt_time > delta_us)) {
+                /* pkt_time_delta has increased, so handle normally */
+                timersub(pkt_time_delta, last_delta, &nap_for);
                 dbgx(3, "original packet delta pkt_time: " TIMEVAL_FORMAT, nap_for.tv_sec, nap_for.tv_usec);
 
                 TIMEVAL_TO_TIMESPEC(&nap_for, &ctx->nap);
                 dbgx(3, "original packet delta timv: " TIMESPEC_FORMAT, ctx->nap.tv_sec, ctx->nap.tv_nsec);
                 timesdiv_float(&ctx->nap, options->speed.multiplier);
                 dbgx(3, "original packet delta/div: " TIMESPEC_FORMAT, ctx->nap.tv_sec, ctx->nap.tv_nsec);
+            } else {
+                /* Don't sleep if this packet is late or in the past */
+                update_time = false;
             }
         } else {
             /* Don't sleep if this is our first packet */
-            timesclear(&ctx->nap);
+            update_time = false;
         }
         break;
 
@@ -1116,11 +1171,19 @@ static void calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
          */
         now_us = TIMSTAMP_TO_MICROSEC(sent_timestamp);
         if (now_us) {
+            COUNTER next_tx_us;
             COUNTER bps = options->speed.speed;
             COUNTER bits_sent = ((ctx->stats.bytes_sent + len) * 8);
-            /* bits * 1000000 divided by bps = microseconds */
-            COUNTER next_tx_us = (bits_sent * 1000000) / bps;
             COUNTER tx_us = now_us - *start_us;
+            /*
+             * bits * 1000000 divided by bps = microseconds
+             *
+             * ensure there is no overflow in cases where bits_sent is very high
+             */
+            if (bits_sent > COUNTER_OVERFLOW_RISK && bps > 500000)
+                next_tx_us = (bits_sent * 1000) / bps * 1000;
+            else
+                next_tx_us = (bits_sent * 1000000) / bps;
             if (next_tx_us > tx_us) {
                 NANOSEC_TO_TIMESPEC((next_tx_us - tx_us) * 1000, &ctx->nap);
             } else if (tx_us > next_tx_us) {
@@ -1141,10 +1204,13 @@ static void calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
           */
          now_us = TIMSTAMP_TO_MICROSEC(sent_timestamp);
          if (now_us) {
-             COUNTER pps = ctx->options->speed.speed * (ctx->options->speed.pps_multi > 0 ? ctx->options->speed.pps_multi : 1);;
+             COUNTER pph = ctx->options->speed.speed * (ctx->options->speed.pps_multi > 0 ? ctx->options->speed.pps_multi : (60 * 60));;
              COUNTER pkts_sent = ctx->stats.pkts_sent;
-             /* packets * 1000000 divided by pps = microseconds */
-             COUNTER next_tx_us = (pkts_sent * 1000000) / pps;
+             /*
+              * packets * 1000000 divided by pps = microseconds
+              * packets per sec (pps) = packets per hour / (60 * 60)
+              */
+             COUNTER next_tx_us = (pkts_sent * 1000000) * (60 * 60) / pph;
              COUNTER tx_us = now_us - *start_us;
              if (next_tx_us > tx_us)
                  NANOSEC_TO_TIMESPEC((next_tx_us - tx_us) * 1000, &ctx->nap);
@@ -1179,6 +1245,8 @@ static void calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
         errx(-1, "Unknown/supported speed mode: %d", options->speed.mode);
         break;
     }
+
+    return update_time;
 }
 
 static void tcpr_sleep(tcpreplay_t *ctx, sendpacket_t *sp,
@@ -1244,7 +1312,7 @@ get_user_count(tcpreplay_t *ctx, sendpacket_t *sp, COUNTER counter)
     tcpreplay_opt_t *options = ctx->options;
     struct pollfd poller[1];        /* use poll to read from the keyboard */
     char input[EBUF_SIZE];
-    uint32_t send = 0;
+    unsigned long send = 0;
 
     printf("**** Next packet #" COUNTER_SPEC " out %s.  How many packets do you wish to send? ",
         counter, (sp == ctx->intf1 ? options->intf1_name : options->intf2_name));
@@ -1280,5 +1348,5 @@ get_user_count(tcpreplay_t *ctx, sendpacket_t *sp, COUNTER counter)
         send = 1;
     }
 
-    return send;
+    return(uint32_t)send;
 }
