@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -49,7 +49,7 @@ do_checksum(tcpedit_t *tcpedit, uint8_t *data, int proto, int len) {
     ipv6 = NULL;
     assert(data);
 
-    if (len <= 0) {
+    if (!data || len <= 0) {
         tcpedit_setwarn(tcpedit, "%s", "Unable to checksum packets with no L3+ data");
         return TCPEDIT_WARN;
     }
@@ -133,9 +133,11 @@ do_checksum(tcpedit_t *tcpedit, uint8_t *data, int proto, int len) {
 
 
         case IPPROTO_IP:
-            ipv4->ip_sum = 0;
-            sum = do_checksum_math((uint16_t *)data, ip_hl);
-            ipv4->ip_sum = CHECKSUM_CARRY(sum);
+            if (ipv4) {
+                ipv4->ip_sum = 0;
+                sum = do_checksum_math((uint16_t *)data, ip_hl);
+                ipv4->ip_sum = CHECKSUM_CARRY(sum);
+            }
             break;
 
         case IPPROTO_IGMP:
