@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -284,13 +284,12 @@ check_ipv4_regex(const unsigned long ip)
     int eflags = 0;
     u_char src_ip[16];
     size_t nmatch = 0;
-    regmatch_t *pmatch = NULL;
     tcpprep_opt_t *options = tcpprep->options;
 
-    memset(src_ip, '\0', 16);
+    memset(src_ip, '\0', sizeof(src_ip));
     strlcpy((char *)src_ip, (char *)get_addr2name4(ip, RESOLVE),
             sizeof(src_ip));
-    if (regexec(&options->preg, (char *)src_ip, nmatch, pmatch, eflags) == 0) {
+    if (regexec(&options->preg, (char *)src_ip, nmatch, NULL, eflags) == 0) {
         return 1;
     } else {
         return 0;
@@ -304,12 +303,12 @@ check_ipv6_regex(const struct tcpr_in6_addr *addr)
     int eflags = 0;
     u_char src_ip[INET6_ADDRSTRLEN];
     size_t nmatch = 0;
-    regmatch_t *pmatch = NULL;
     tcpprep_opt_t *options = tcpprep->options;
 
-    memset(src_ip, '\0', 16);
-    strlcpy((char *)src_ip, (char *)get_addr2name6(addr, RESOLVE), sizeof(src_ip));
-    if (regexec(&options->preg, (char *)src_ip, nmatch, pmatch, eflags) == 0) {
+    memset(src_ip, '\0', sizeof(src_ip));
+    strlcpy((char *)src_ip, (char *)get_addr2name6(addr, RESOLVE),
+        sizeof(src_ip));
+    if (regexec(&options->preg, (char *)src_ip, nmatch, NULL, eflags) == 0) {
         return 1;
     } else {
         return 0;
